@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.16 2003/04/27 21:10:51 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.29 2003/05/11 15:57:37 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -60,21 +60,26 @@ enum {
 	ERROR,
 	EOS,
 	TITLE_CHANGE,
+	CHANNELS_CHANGE,
 	TICK,
+	GOT_METADATA,
+	BUFFERING,
 	LAST_SIGNAL
 };
 
 /* Enum for none-signal stuff that needs to go through the AsyncQueue */
 enum {
 	RATIO,
-	PROGRESS,
 	TITLE_CHANGE_ASYNC,
 	EOS_ASYNC,
+	CHANNELS_CHANGE_ASYNC,
+	BUFFERING_ASYNC
 };
 
 typedef struct {
 	int signal;
 	char *msg;
+	int num;
 } signal_data;
 
 /* Arguments */
@@ -106,9 +111,11 @@ struct BaconVideoWidgetPrivate {
 	xine_ao_driver_t *ao_driver;
 	xine_event_queue_t *ev_queue;
 	double display_ratio;
+	gboolean started;
 
 	/* Configuration */
 	gboolean null_out;
+	char *mrl;
 
 	/* X stuff */
 	Display *display;
@@ -116,10 +123,11 @@ struct BaconVideoWidgetPrivate {
 	GdkWindow *video_window;
 
 	/* Visual effects */
-	char *mrl;
+	char *vis_name;
 	gboolean show_vfx;
 	gboolean using_vfx;
 	xine_post_t *vis;
+	GList *visuals;
 
 	/* Other stuff */
 	int xpos, ypos;
@@ -292,6 +300,14 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				G_STRUCT_OFFSET (BaconVideoWidgetClass, eos),
 				NULL, NULL,
 				g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+				
+	bvw_table_signals[GOT_METADATA] =
+		g_signal_new ("got-metadata",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (BaconVideoWidgetClass, got_metadata),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	bvw_table_signals[TITLE_CHANGE] =
 		g_signal_new ("title-change",
@@ -301,6 +317,16 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__STRING,
 				G_TYPE_NONE, 1, G_TYPE_STRING);
+
+	bvw_table_signals[CHANNELS_CHANGE] =
+		g_signal_new ("channels-change",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (BaconVideoWidgetClass, channels_change),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
+
 	bvw_table_signals[TICK] =
 		g_signal_new ("tick",
 				G_TYPE_FROM_CLASS (object_class),
@@ -310,6 +336,15 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				baconvideowidget_marshal_VOID__INT_INT_INT,
 				G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_INT,
 				G_TYPE_INT);
+
+	bvw_table_signals[BUFFERING] =
+		g_signal_new ("buffering",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (BaconVideoWidgetClass, buffering),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__INT,
+				G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
 static void
@@ -332,6 +367,7 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	bvw->priv = g_new0 (BaconVideoWidgetPrivate, 1);
 	bvw->priv->xine = xine_new ();
 	bvw->priv->cursor_shown = TRUE;
+	bvw->priv->vis_name = g_strdup ("goom");
 
 	bvw->priv->queue = g_async_queue_new ();
 
@@ -374,6 +410,9 @@ bacon_video_widget_finalize (GObject *object)
 {
 	BaconVideoWidget *bvw = (BaconVideoWidget *) object;
 
+	g_list_foreach (bvw->priv->visuals, (GFunc) g_free, NULL);
+	g_list_free (bvw->priv->visuals);
+
 	if (bvw->priv->icon != NULL)
 		gdk_pixbuf_unref (bvw->priv->icon);
 
@@ -643,6 +682,20 @@ setup_config (BaconVideoWidget *bvw)
 	bvw->priv->volume = -1;
 }
 
+static void
+setup_config_video (BaconVideoWidget *bvw)
+{
+	xine_cfg_entry_t entry;
+
+	/* Default xv colourkey */
+	if (xine_config_lookup_entry (bvw->priv->xine,
+				"video.xv_colorkey", &entry))
+	{
+		entry.num_value = 30;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
+}
+
 static gboolean
 video_window_translate_point (BaconVideoWidget *bvw, int gui_x, int gui_y,
 		int *video_x, int *video_y)
@@ -850,6 +903,11 @@ bacon_video_widget_realize (GtkWidget *widget)
 
 	bvw->priv->video_window = widget->window;
 
+	/* Set a black background */
+	gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE,
+			attr.x, attr.y,
+			attr.width, attr.height);
+
 	/* track configure events of toplevel window */
 	g_signal_connect (G_OBJECT (gtk_widget_get_toplevel (widget)),
 			"configure-event",
@@ -863,11 +921,13 @@ bacon_video_widget_realize (GtkWidget *widget)
 	bvw->priv->screen = DefaultScreen (bvw->priv->display);
 
 	bvw->priv->vo_driver = load_video_out_driver (bvw, bvw->priv->null_out);
+	setup_config_video (bvw);
 	//FIXME
 	g_assert (bvw->priv->vo_driver != NULL);
 
-	if (bvw->priv->null_out == FALSE)
-		bvw->priv->vis = xine_post_init (bvw->priv->xine, "goom", 0,
+	if (bvw->priv->null_out == FALSE && bvw->priv->ao_driver != NULL)
+		bvw->priv->vis = xine_post_init (bvw->priv->xine,
+				bvw->priv->vis_name, 0,
 				&bvw->priv->ao_driver, &bvw->priv->vo_driver);
 
 	bvw->priv->stream = xine_stream_new (bvw->priv->xine,
@@ -903,12 +963,6 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 	case RATIO:
 		bacon_video_widget_set_scale_ratio (bvw, 0);
 		break;
-	case PROGRESS:
-		g_message ("%s", data->msg);
-		while (gtk_events_pending ())
-			gtk_main_iteration ();
-		//FIXME
-		break;
 	case TITLE_CHANGE_ASYNC:
 		g_signal_emit (G_OBJECT (bvw),
 				bvw_table_signals[TITLE_CHANGE],
@@ -918,6 +972,15 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 		g_signal_emit (G_OBJECT (bvw),
 				bvw_table_signals[EOS], 0, NULL);
 		break;
+	case CHANNELS_CHANGE_ASYNC:
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[CHANNELS_CHANGE], 0, NULL);
+		break;
+	case BUFFERING_ASYNC:
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[BUFFERING],
+				0, data->num);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -948,6 +1011,12 @@ xine_event (void *user_data, const xine_event_t *event)
 		g_async_queue_push (bvw->priv->queue, data);
 		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
 		break;
+	case XINE_EVENT_UI_CHANNELS_CHANGED:
+		data = g_new0 (signal_data, 1);
+		data->signal = CHANNELS_CHANGE_ASYNC;
+		g_async_queue_push (bvw->priv->queue, data);
+		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
+		break;
 	case XINE_EVENT_UI_SET_TITLE:
 		ui_data = event->data;
 
@@ -961,9 +1030,8 @@ xine_event (void *user_data, const xine_event_t *event)
 		prg = event->data;
 
 		data = g_new0 (signal_data, 1);
-		data->signal = PROGRESS;
-		data->msg = g_strdup_printf ("%s %d%%",
-				prg->description, prg->percent);
+		data->signal = BUFFERING_ASYNC;
+		data->num = prg->percent;
 		g_async_queue_push (bvw->priv->queue, data);
 		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
 		break;
@@ -1121,6 +1189,13 @@ bacon_video_widget_expose (GtkWidget *widget, GdkEventExpose *event)
 	if (event->count != 0)
 		return FALSE;
 
+	if (bvw->priv->started == FALSE)
+	{
+		gdk_draw_rectangle (widget->window, widget->style->black_gc,
+				TRUE, event->area.x, event->area.y,
+				event->area.width, event->area.height);
+	}
+
 	expose = g_new0 (XExposeEvent, 1);
 	expose->count = event->count;
 
@@ -1293,7 +1368,6 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
 		GError **gerror)
 {
 	int error;
-	gboolean has_video;
 
 	g_return_val_if_fail (bvw != NULL, FALSE);
 	g_return_val_if_fail (mrl != NULL, FALSE);
@@ -1321,6 +1395,9 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
 		uint32_t fourcc;
 		char *fourcc_str, *name;
 
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[GOT_METADATA], 0, NULL);
+
 		fourcc = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_FOURCC);
 		fourcc_str = get_fourcc_string (fourcc);
@@ -1343,6 +1420,9 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
 				XINE_STREAM_INFO_HAS_VIDEO) == FALSE
 		&& bvw->priv->ao_driver == NULL)
 	{
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[GOT_METADATA], 0, NULL);
+
 		bacon_video_widget_close (bvw);
 
 		g_set_error (gerror, 0, 0,
@@ -1353,6 +1433,9 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const gchar *mrl,
 
 	show_vfx_update (bvw, bvw->priv->show_vfx);
 
+	g_signal_emit (G_OBJECT (bvw),
+			bvw_table_signals[GOT_METADATA], 0, NULL);
+
 	return TRUE;
 }
 
@@ -1366,6 +1449,8 @@ bacon_video_widget_play (BaconVideoWidget *bvw, guint pos,
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
 	g_return_val_if_fail (bvw->priv->xine != NULL, -1);
 
+	bvw->priv->started = TRUE;
+
 	length = bacon_video_widget_get_stream_length (bvw);
 
 	error = xine_play (bvw->priv->stream, pos,
@@ -1821,11 +1906,15 @@ bacon_video_widget_set_proprietary_plugins_path (BaconVideoWidget *bvw,
 
 	memset (&entry, 0, sizeof (entry));
 	if (xine_config_lookup_entry (bvw->priv->xine,
-				"codec.win32_path", &entry))
+				"codec.real_codecs_path", &entry))
 	{
 		entry.str_value = g_strdup (path);
 		xine_config_update_entry (bvw->priv->xine, &entry);
 	}
+
+	/* And we try and create symlinks from /usr/lib/win32 to
+	 * the local user path */
+	totem_create_symlinks ("/usr/lib/win32", path);
 }
 
 void
@@ -1855,7 +1944,6 @@ int
 bacon_video_widget_get_connection_speed (BaconVideoWidget *bvw)
 {
 	xine_cfg_entry_t entry;
-	int i = 0;
 
 	g_return_val_if_fail (bvw != NULL, 0);
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
@@ -1899,9 +1987,9 @@ bacon_video_widget_get_deinterlacing (BaconVideoWidget *bvw)
 gboolean
 bacon_video_widget_set_tv_out (BaconVideoWidget *bvw, TvOutType tvout)
 {
-	g_return_if_fail (bvw != NULL);
-	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-	g_return_if_fail (bvw->priv->xine != NULL);
+	g_return_val_if_fail (bvw != NULL, FALSE);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 
 	if (tvout == TV_OUT_DXR3 || bvw->priv->tvout == TV_OUT_DXR3)
 	{
@@ -1943,13 +2031,115 @@ gboolean
 bacon_video_widget_set_show_visuals (BaconVideoWidget *bvw,
 		gboolean show_visuals)
 {
+	g_return_val_if_fail (bvw != NULL, FALSE);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
+
+	bvw->priv->show_vfx = show_visuals;
+	show_vfx_update (bvw, show_visuals);
+
+	return TRUE;
+}
+
+GList *
+bacon_video_widget_get_visuals_list (BaconVideoWidget *bvw)
+{
+	const char * const* plugins;
+	int i;
+
+	g_return_val_if_fail (bvw != NULL, NULL);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
+	g_return_val_if_fail (bvw->priv->xine != NULL, NULL);
+
+	if (bvw->priv->visuals != NULL)
+		return bvw->priv->visuals;
+
+	plugins = xine_list_post_plugins_typed (bvw->priv->xine,
+			XINE_POST_TYPE_AUDIO_VISUALIZATION);
+
+	for (i = 0; plugins[i] != NULL; i++)
+	{
+		bvw->priv->visuals = g_list_prepend
+			(bvw->priv->visuals, g_strdup (plugins[i]));
+	}
+
+	bvw->priv->visuals = g_list_reverse (bvw->priv->visuals);
+
+	return bvw->priv->visuals;
+}
+
+gboolean
+bacon_video_widget_set_visuals (BaconVideoWidget *bvw, const char *name)
+{
+	g_return_val_if_fail (bvw != NULL, FALSE);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
+
+	g_free (bvw->priv->vis_name);
+	bvw->priv->vis_name = g_strdup (name);
+
+	if (GTK_WIDGET_REALIZED (GTK_WIDGET (bvw)))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+void
+bacon_video_widget_set_visuals_quality (BaconVideoWidget *bvw,
+		VisualsQuality quality)
+{
+	xine_cfg_entry_t entry;
+	int fps, w, h;
+
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
-	bvw->priv->show_vfx = show_visuals;
-	show_vfx_update (bvw, show_visuals);
-	return TRUE;
+	switch (quality)
+	{
+	case VISUAL_SMALL:
+		fps = 25;
+		w = 160;
+		h = 120;
+		break;
+	case VISUAL_NORMAL:
+		fps = 15;
+		w = 320;
+		h = 240;
+		break;
+	case VISUAL_LARGE:
+		fps = 25;
+		w = 640;
+		h = 480;
+		break;
+	case VISUAL_EXTRA_LARGE:
+		fps = 25;
+		w = 1024;
+		h = 768;
+		break;
+	default:
+		/* shut up warnings */
+		fps = w = h = 0;
+		g_assert_not_reached ();
+	}
+
+	if (xine_config_lookup_entry (bvw->priv->xine, "post.goom_fps", &entry))
+	{
+		entry.num_value = fps;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
+
+	if (xine_config_lookup_entry (bvw->priv->xine, "post.goom_width", &entry))
+	{
+		entry.num_value = w;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
+
+	if (xine_config_lookup_entry (bvw->priv->xine, "post.goom_height", &entry))
+	{
+		entry.num_value = h;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
 }
 
 void
@@ -2354,7 +2544,7 @@ GList
 *bacon_video_widget_get_languages (BaconVideoWidget *bvw)
 {
 	GList *list = NULL;
-	int retval, i;
+	int i;
 	char lang[16];
 
 	for(i = 0; i < 32; i++)
@@ -2387,7 +2577,7 @@ GList
 *bacon_video_widget_get_subtitles (BaconVideoWidget *bvw)
 {
 	GList *list = NULL;
-	int retval, i;
+	int i;
 	char lang[16];
 
 	for(i = 0; i < 32; i++)
