@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget.h,v 1.6 2003/02/22 00:25:59 hadess Exp $
+ * $Id: bacon-video-widget.h,v 1.22 2003/05/11 13:30:29 hadess Exp $
  *
  * the xine engine in a widget - header
  */
@@ -24,7 +24,7 @@
 #ifndef HAVE_BACON_VIDEO_WIDGET_H
 #define HAVE_BACON_VIDEO_WIDGET_H
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtkvbox.h>
 
 G_BEGIN_DECLS
 
@@ -33,27 +33,18 @@ typedef enum {
 	SPEED_NORMAL,
 } Speeds;
 
-typedef enum {
-	BVW_DVD_ROOT_MENU,
-	BVW_DVD_TITLE_MENU,
-	BVW_DVD_SUBPICTURE_MENU,
-	BVW_DVD_AUDIO_MENU,
-	BVW_DVD_ANGLE_MENU,
-	BVW_DVD_CHAPTER_MENU,
-	BVW_DVD_NEXT_CHAPTER,
-	BVW_DVD_PREV_CHAPTER,
-	BVW_DVD_NEXT_TITLE,
-	BVW_DVD_PREV_TITLE,
-	BVW_DVD_NEXT_ANGLE,
-	BVW_DVD_PREV_ANGLE
-} BaconVideoWidgetDVDEvent;
-
 typedef enum {
 	MEDIA_DVD,
 	MEDIA_VCD,
 	MEDIA_CDDA,
 } MediaType;
 
+typedef enum {
+	TV_OUT_NONE,
+	TV_OUT_DXR3,
+	TV_OUT_TVMODE,
+} TvOutType;
+
 typedef enum {
 	BVW_INFO_TITLE,
 	BVW_INFO_ARTIST,
@@ -79,18 +70,21 @@ typedef enum {
 typedef struct BaconVideoWidgetPrivate BaconVideoWidgetPrivate;
 
 typedef struct {
-	GtkWidget widget;
+	GtkVBox parent;
 	BaconVideoWidgetPrivate *priv;
 } BaconVideoWidget;
 
 typedef struct {
-	GtkWidgetClass parent_class;
+	GtkVBoxClass parent_class;
 
 	void (*error) (GtkWidget *bvw, const char *message);
 	void (*eos) (GtkWidget *bvw);
+	void (*got_metadata) (GtkWidget *bvw);
 	void (*title_change) (GtkWidget *bvw, const char *title);
+	void (*channels_change) (GtkWidget *bvw);
 	void (*tick) (GtkWidget *bvw, int current_time, int stream_length,
 			int current_position);
+	void (*buffering) (GtkWidget *bvw, guint progress);
 } BaconVideoWidgetClass;
 
 GType bacon_video_widget_get_type                (void);
@@ -98,6 +92,8 @@ GtkWidget *bacon_video_widget_new		 (int width, int height,
 						  gboolean null_video_out,
 						  GError **error);
 
+char *bacon_video_widget_get_backend_name (BaconVideoWidget *bvw);
+
 /* Actions */
 gboolean bacon_video_widget_open		 (BaconVideoWidget *bvw,
 						  const gchar *mrl,
@@ -112,18 +108,22 @@ gboolean bacon_video_widget_play                 (BaconVideoWidget *bvw,
 						  GError **error);
 void bacon_video_widget_stop                     (BaconVideoWidget *bvw);
 void bacon_video_widget_close                    (BaconVideoWidget *bvw);
-
-void bacon_video_widget_dvd_event                (BaconVideoWidget *bvw,
-						  BaconVideoWidgetDVDEvent type);
-
+gboolean bacon_video_widget_eject                (BaconVideoWidget *bvw);
+void bacon_video_widget_set_logo                 (BaconVideoWidget *bvw,
+                                                  gchar *filename);
 /* Properties */
 void  bacon_video_widget_set_logo_mode		 (BaconVideoWidget *bvw,
 						  gboolean logo_mode);
 gboolean bacon_video_widget_get_logo_mode	 (BaconVideoWidget *bvw);
 
-void bacon_video_widget_set_speed                (BaconVideoWidget *bvw, Speeds speed);
+void bacon_video_widget_set_speed                (BaconVideoWidget *bvw,
+		                                  Speeds speed);
 int bacon_video_widget_get_speed                 (BaconVideoWidget *bvw);
 
+void bacon_video_widget_set_proprietary_plugins_path
+						 (BaconVideoWidget *bvw,
+				                  const char *path);
+
 void bacon_video_widget_set_fullscreen           (BaconVideoWidget *bvw,
 						  gboolean fullscreen);
 gboolean bacon_video_widget_is_fullscreen        (BaconVideoWidget *bvw);
@@ -139,11 +139,21 @@ gboolean bacon_video_widget_get_show_cursor      (BaconVideoWidget *bvw);
 
 void bacon_video_widget_set_media_device	 (BaconVideoWidget *bvw,
 						  const char *path);
-void bacon_video_widget_set_show_visuals	 (BaconVideoWidget *bvw,
-						  gboolean show_visuals);
 void bacon_video_widget_set_auto_resize		 (BaconVideoWidget *bvw,
 						  gboolean auto_resize);
 
+void bacon_video_widget_set_connection_speed     (BaconVideoWidget *bvw,
+						  int speed);
+int bacon_video_widget_get_connection_speed      (BaconVideoWidget *bvw);
+
+void bacon_video_widget_set_deinterlacing	 (BaconVideoWidget *bvw,
+						  gboolean deinterlace);
+gboolean bacon_video_widget_get_deinterlacing	 (BaconVideoWidget *bvw);
+
+gboolean bacon_video_widget_set_tv_out		 (BaconVideoWidget *bvw,
+						  TvOutType tvout);
+TvOutType bacon_video_widget_get_tv_out		 (BaconVideoWidget *bvw);
+
 void bacon_video_widget_toggle_aspect_ratio      (BaconVideoWidget *bvw);
 void bacon_video_widget_set_scale_ratio          (BaconVideoWidget *bvw,
 						  gfloat ratio);
@@ -161,9 +171,55 @@ G_CONST_RETURN gchar **bacon_video_widget_get_mrls
 						  MediaType type);
 
 void bacon_video_widget_get_metadata		 (BaconVideoWidget *bvw,
-						  BaconVideoWidgetMetadataType type,
+						  BaconVideoWidgetMetadataType
+						  type,
 						  GValue *value);
 
+/* Visualisation functions */
+typedef enum {
+	VISUAL_SMALL,
+	VISUAL_NORMAL,
+	VISUAL_LARGE,
+	VISUAL_EXTRA_LARGE,
+} VisualsQuality;
+
+gboolean bacon_video_widget_set_show_visuals	  (BaconVideoWidget *bvw,
+						   gboolean show_visuals);
+GList *bacon_video_widget_get_visuals_list	  (BaconVideoWidget *bvw);
+gboolean bacon_video_widget_set_visuals		  (BaconVideoWidget *bvw,
+						   const char *name);
+void bacon_video_widget_set_visuals_quality	  (BaconVideoWidget *bvw,
+						   VisualsQuality quality);
+
+/* DVD functions */
+typedef enum {
+	BVW_DVD_ROOT_MENU,
+	BVW_DVD_TITLE_MENU,
+	BVW_DVD_SUBPICTURE_MENU,
+	BVW_DVD_AUDIO_MENU,
+	BVW_DVD_ANGLE_MENU,
+	BVW_DVD_CHAPTER_MENU,
+	BVW_DVD_NEXT_CHAPTER,
+	BVW_DVD_PREV_CHAPTER,
+	BVW_DVD_NEXT_TITLE,
+	BVW_DVD_PREV_TITLE,
+	BVW_DVD_NEXT_ANGLE,
+	BVW_DVD_PREV_ANGLE
+} BaconVideoWidgetDVDEvent;
+
+void bacon_video_widget_dvd_event                (BaconVideoWidget *bvw,
+						  BaconVideoWidgetDVDEvent
+						  type);
+GList *bacon_video_widget_get_languages          (BaconVideoWidget *bvw);
+int bacon_video_widget_get_language              (BaconVideoWidget *bvw);
+void bacon_video_widget_set_language             (BaconVideoWidget *bvw,
+		                                  int language);
+
+GList *bacon_video_widget_get_subtitles          (BaconVideoWidget *bvw);
+int bacon_video_widget_get_subtitle              (BaconVideoWidget *bvw);
+void bacon_video_widget_set_subtitle             (BaconVideoWidget *bvw,
+		                                  int subtitle);
+
 /* Screenshot functions */
 gboolean bacon_video_widget_can_get_frames       (BaconVideoWidget *bvw,
 						  GError **error);
